import { __rest } from 'tslib';
import * as fs from 'fs';
import * as path from 'path';
import { resolveSnapshotFile } from '@component-controls/core/node-utils';
import { getOptions } from 'loader-utils';
import { log } from '@component-controls/logger';
import { deepmerge } from '@component-controls/core';
import { parseStories } from '@component-controls/instrument';
import { store, addStoriesDoc, removeStoriesDoc } from './store.esm.js';
import '@component-controls/config';

async function loader() {
    var _a, _b;
    const filePath = this.resource;
    const loaderOptions = getOptions(this) || {};
    const configOptions = ((_a = store === null || store === void 0 ? void 0 : store.buildConfig) === null || _a === void 0 ? void 0 : _a.instrument) || {};
    const ignore = ((_b = store === null || store === void 0 ? void 0 : store.buildConfig) === null || _b === void 0 ? void 0 : _b.ignore) || [];
    const basePath = path.basename(filePath).toLowerCase();
    const source = fs.readFileSync(filePath, 'utf8');
    if (!ignore.includes(basePath)) {
        const options = deepmerge(configOptions, loaderOptions);
        const _c = await parseStories(filePath, source, options), { transformed } = _c, store = __rest(_c, ["transformed"]);
        if (store === null || store === void 0 ? void 0 : store.doc) {
            log('loaded: ', filePath);
            const fileDependencies = [];
            if (store.doc.testData) {
                fileDependencies.push(path.resolve(path.dirname(filePath), store.doc.testData));
            }
            if (store.stories && store.components && store.packages) {
                Object.values(store.components).forEach(component => {
                    if (component.request) {
                        fileDependencies.push(component.request);
                        if (component.jest) {
                            const componentFolder = path.dirname(component.request);
                            component.jest.results.forEach(r => {
                                const testFilePath = path.resolve(componentFolder, r.testFilePath);
                                fileDependencies.push(testFilePath);
                                const snapshotFile = resolveSnapshotFile(testFilePath);
                                fileDependencies.push(snapshotFile);
                            });
                            Object.keys(component.jest.coverage).forEach(f => {
                                fileDependencies.push(path.resolve(componentFolder, f));
                            });
                        }
                    }
                });
                addStoriesDoc(filePath, this._compilation.records.hash, {
                    stories: store.stories,
                    components: store.components,
                    packages: store.packages,
                    doc: Object.assign(Object.assign({}, store.doc), { fileName: filePath }),
                });
            }
            new Set(fileDependencies).forEach(d => {
                this.addDependency(d);
            });
        }
        else {
            log('removed: ', filePath);
            removeStoriesDoc(filePath);
        }
        return transformed;
    }
    return source;
}

export { loader as default };
