'use strict';

var tslib = require('tslib');
var fs = require('fs');
var path = require('path');
var nodeUtils = require('@component-controls/core/node-utils');
var loaderUtils = require('loader-utils');
var logger = require('@component-controls/logger');
var core = require('@component-controls/core');
var instrument = require('@component-controls/instrument');
var store = require('./store.js');
require('@component-controls/config');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n["default"] = e;
  return Object.freeze(n);
}

var fs__namespace = /*#__PURE__*/_interopNamespace(fs);
var path__namespace = /*#__PURE__*/_interopNamespace(path);

async function loader() {
    var _a, _b;
    const filePath = this.resource;
    const loaderOptions = loaderUtils.getOptions(this) || {};
    const configOptions = ((_a = store.store === null || store.store === void 0 ? void 0 : store.store.buildConfig) === null || _a === void 0 ? void 0 : _a.instrument) || {};
    const ignore = ((_b = store.store === null || store.store === void 0 ? void 0 : store.store.buildConfig) === null || _b === void 0 ? void 0 : _b.ignore) || [];
    const basePath = path__namespace.basename(filePath).toLowerCase();
    const source = fs__namespace.readFileSync(filePath, 'utf8');
    if (!ignore.includes(basePath)) {
        const options = core.deepmerge(configOptions, loaderOptions);
        const _c = await instrument.parseStories(filePath, source, options), { transformed } = _c, store$1 = tslib.__rest(_c, ["transformed"]);
        if (store$1 === null || store$1 === void 0 ? void 0 : store$1.doc) {
            logger.log('loaded: ', filePath);
            const fileDependencies = [];
            if (store$1.doc.testData) {
                fileDependencies.push(path__namespace.resolve(path__namespace.dirname(filePath), store$1.doc.testData));
            }
            if (store$1.stories && store$1.components && store$1.packages) {
                Object.values(store$1.components).forEach(component => {
                    if (component.request) {
                        fileDependencies.push(component.request);
                        if (component.jest) {
                            const componentFolder = path__namespace.dirname(component.request);
                            component.jest.results.forEach(r => {
                                const testFilePath = path__namespace.resolve(componentFolder, r.testFilePath);
                                fileDependencies.push(testFilePath);
                                const snapshotFile = nodeUtils.resolveSnapshotFile(testFilePath);
                                fileDependencies.push(snapshotFile);
                            });
                            Object.keys(component.jest.coverage).forEach(f => {
                                fileDependencies.push(path__namespace.resolve(componentFolder, f));
                            });
                        }
                    }
                });
                store.addStoriesDoc(filePath, this._compilation.records.hash, {
                    stories: store$1.stories,
                    components: store$1.components,
                    packages: store$1.packages,
                    doc: Object.assign(Object.assign({}, store$1.doc), { fileName: filePath }),
                });
            }
            new Set(fileDependencies).forEach(d => {
                this.addDependency(d);
            });
        }
        else {
            logger.log('removed: ', filePath);
            store.removeStoriesDoc(filePath);
        }
        return transformed;
    }
    return source;
}

module.exports = loader;
