'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var path = require('path');
var webpack = require('webpack');
var crypto = require('crypto');
var jsStringEscape = require('js-string-escape');
var store = require('./store.js');
require('@component-controls/core');
require('@component-controls/config');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n["default"] = e;
  return Object.freeze(n);
}

var path__namespace = /*#__PURE__*/_interopNamespace(path);
var webpack__namespace = /*#__PURE__*/_interopNamespace(webpack);
var jsStringEscape__default = /*#__PURE__*/_interopDefaultLegacy(jsStringEscape);

class LoaderPlugin {
    constructor(options) {
        const hash = crypto.createHash('md5')
            .update(new Date().getTime().toString())
            .digest('hex');
        this.compilationHash = `__${hash.substring(0, 5)}__`;
        this.options = Object.assign({}, options);
    }
    apply(compiler) {
        store.initializeBuildOptions(compiler.context, this.options.config, this.options.defaultConfigPath, this.options.buildConfig);
        this.replaceRuntimeModule(compiler);
        compiler.hooks.compilation.tap(LoaderPlugin.pluginName, compilation => {
            var _a, _b;
            if (typeof ((_a = compilation.hooks.processAssets) === null || _a === void 0 ? void 0 : _a.tapPromise) === 'function') {
                compilation.hooks.processAssets.tapPromise({
                    name: LoaderPlugin.pluginName,
                    stage: webpack__namespace.Compilation.PROCESS_ASSETS_STAGE_ADDITIONS,
                }, async (assets) => {
                    const jsFiles = [];
                    Object.keys(assets)
                        .filter(fileName => {
                        return fileName.endsWith('.js');
                    })
                        .forEach(fileName => jsFiles.push(fileName));
                    for (let i = 0; i < jsFiles.length; i += 1) {
                        await this.replaceSource(compilation, jsFiles[i]);
                    }
                });
            }
            else if (typeof ((_b = compilation.hooks.optimizeChunkAssets) === null || _b === void 0 ? void 0 : _b.tapPromise) === 'function') {
                compilation.hooks.optimizeChunkAssets.tapPromise(LoaderPlugin.pluginName, async (chunks) => {
                    const jsFiles = [];
                    chunks.forEach(chunk => {
                        chunk.files
                            .filter((fileName) => fileName.endsWith('.js'))
                            .forEach((fileName) => jsFiles.push(fileName));
                    });
                    for (let i = 0; i < jsFiles.length; i += 1) {
                        await this.replaceSource(compilation, jsFiles[i]);
                    }
                });
            }
        });
    }
    replaceRuntimeModule(compiler) {
        const nmrp = new webpack__namespace.NormalModuleReplacementPlugin(/story-store-data\.js$/, (resource) => {
            var _a;
            //webpack 4 or webpack 5
            const plugins = resource.loaders || ((_a = resource.createData) === null || _a === void 0 ? void 0 : _a.loaders);
            if (plugins) {
                plugins.push({
                    loader: path__namespace.join(__dirname, 'runtimeLoader.js'),
                    options: JSON.stringify({
                        compilationHash: this.compilationHash,
                    }),
                });
            }
        });
        nmrp.apply(compiler);
    }
    async replaceSource(compilation, file) {
        const placeholder = `__COMPILATION_HASH__${this.compilationHash}`;
        const source = compilation.assets[file];
        const sources = webpack__namespace.sources || require('webpack-sources');
        const placeholderPos = source.source().indexOf(placeholder);
        if (placeholderPos > -1) {
            const store$1 = await store.getSerializedStore();
            const newContent = this.options.escapeOutput
                ? jsStringEscape__default["default"](store$1)
                : store$1;
            const newSource = new sources.ReplaceSource(source, file);
            newSource.replace(placeholderPos, placeholderPos + placeholder.length - 1, newContent, file);
            compilation.assets[file] = newSource;
        }
    }
}
LoaderPlugin.pluginName = 'component-controls-loader-plugin';
module.exports = LoaderPlugin;

exports.LoaderPlugin = LoaderPlugin;
