'use strict';

var config = require('@component-controls/config');
var core = require('@component-controls/core');
var store = require('./store.js');

const replaceSource = (contexts, configFilePath, config, hashKey) => {
    var _a;
    const imports = `

const configJSON = ${configFilePath ? `require("${core.normalizePath(configFilePath)}")` : 'undefined'};

const search = ${typeof ((_a = config === null || config === void 0 ? void 0 : config.search) === null || _a === void 0 ? void 0 : _a.searchingModule) === 'string'
        ? `require("${core.normalizePath(config.search.searchingModule)}")`
        : 'undefined'};
${(config === null || config === void 0 ? void 0 : config.pages)
        ? Object.keys(config.pages)
            .map(key => {
            const { tabs } = config.pages[key];
            return tabs
                ? Object.keys(tabs)
                    .map((route, index) => {
                    const tab = tabs[route];
                    const m = Array.isArray(tab) ? tab[0] : tab;
                    return typeof m === 'string'
                        ? `const ${key}_${index} = require("${m}");`
                        : undefined;
                })
                    .filter(tab => tab)
                    .join('\n')
                : undefined;
        })
            .filter(page => page)
            .join('') || ''
        : ''}
const contexts = [];
${contexts
        .map(context => `contexts.push({
        folder: "${core.normalizePath(context.directory)}", 
        req: require.context('${core.normalizePath(context.directory)}', ${context.useSubdirectories ? 'true' : 'false'}, ${context.regExp})
    });`)
        .join('\n')}
`;
    const storeConst = `const store = ${hashKey};\n`;
    const loadStories = `
  
  for (let i = 0; i < store.stores.length; i+= 1) {
    const s =  store.stores[i];
    const doc = s.doc;
    if (doc) {
      let exports = null;
      for (const context of contexts) {
        const key = context.req.keys().find(k => {
          const fullPath = path.join(context.folder, k);
          const normalizedPath = context.folder.indexOf('\\\\') >= 0 ? fullPath.split('/').join('\\\\') : fullPath;
          return doc.fileName === normalizedPath;
        });
        if (key) {
          exports = context.req(key);
          break;
        }
      }
      if (exports) {
        try {
          if (exports.default) {
            const { storySource, ...rest } = exports.default;
            assignProps(doc, rest);
          }
          Object.keys(exports).filter(key => key !== 'default').forEach(key => {
            const exported = exports[key];
            const story = s.stories[key];
            if (story) {
              story.storyFn = typeof exported === 'function' ? exported : (doc.template || exported);
              assignProps(story, exported);
              if (exported.story) {
                assignProps(story, exported.story);
              }
            }
          });
        } catch (e) {
          console.error(\`unable to load module \${doc.moduleId}\`, e);
        }
      } else {
        //file could not be found
        store.stores.splice(i, 1);
        i -= 1; 
      }
    }  
  }
`;
    const exports = `module.exports = store;\n`;
    const newContent = `

const path = require('path-browserify');
const { loadPageTab, assignProps } = require('@component-controls/core')
${imports}
${storeConst}
store.search = search.default || search;
store.config = ${configFilePath ? 'configJSON.default ||' : ''} configJSON;
store.buildConfig = ${config ? JSON.stringify(config) : '{}'};
${(config === null || config === void 0 ? void 0 : config.pages)
        ? Object.keys(config.pages)
            .map(key => {
            const { tabs = {} } = config.pages[key];
            return (Object.keys(tabs)
                .map((tab, index) => `store.buildConfig.pages.${key}.tabs["${tab}"] = 
  loadPageTab(store.buildConfig.pages.${key}.tabs["${tab}"], ${key}_${index});`)
                .filter(template => template)
                .join('\n') || undefined);
        })
            .filter(page => page)
            .join('') || ''
        : ''}
${loadStories}
${exports}
`;
    return newContent;
};

function loader(content) {
    const params = JSON.parse(this.query.slice(1));
    const contexts = store.config ? config.configRequireContext(store.config) || [] : [];
    const stories = store.config ? config.extractDocuments(store.config) || [] : [];
    store.reserveStories(stories);
    content = replaceSource(contexts, store.config === null || store.config === void 0 ? void 0 : store.config.optionsFilePath, store.store.buildConfig, `__COMPILATION_HASH__${params.compilationHash}`);
    return content;
}

module.exports = loader;
