'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var core = require('@component-controls/core');
var config = require('@component-controls/config');

const store = {
    stores: [],
    components: {},
    packages: {},
    config: {},
    buildConfig: {},
    search: () => {
        return {};
    },
};
const reserveStories = (filePaths) => {
    if (store.stores.length === 0) {
        filePaths.forEach(filePath => store.stores.push({ filePath }));
    }
};
const removeStoriesDoc = (filePath) => {
    store.stores = store.stores.filter(s => s.filePath !== filePath);
};
const addStoriesDoc = (filePath, hash, added) => {
    const { components, packages, stories, doc } = added;
    if (!doc) {
        throw new Error(`Invalid store with no document ${filePath}`);
    }
    Object.keys(components).forEach(key => {
        store.components[key] = components[key];
    });
    Object.keys(packages).forEach(key => {
        store.packages[key] = packages[key];
    });
    const { title } = doc;
    if (store.stores.find(s => { var _a; return s.filePath !== filePath && ((_a = s.doc) === null || _a === void 0 ? void 0 : _a.title) === title; })) {
        throw new Error(`Duplicate document title "${title}"`);
    }
    const storeStore = store.stores.find(s => s.filePath === filePath);
    if (storeStore) {
        storeStore.stories = stories;
        storeStore.doc = doc;
        storeStore.hash = hash;
    }
    else {
        store.stores.push({ filePath, hash, stories, doc });
    }
};
const getSerializedStore = async () => {
    return JSON.stringify(store);
};
exports.config = void 0;
const initializeBuildOptions = (rootPath, configPath, defaultConfigPath, userBuildConfig) => {
    exports.config = config.loadConfiguration(rootPath, configPath, undefined, defaultConfigPath);
    const buildConfig = userBuildConfig
        ? core.mergeConfig(core.defaultBuildConfig, userBuildConfig)
        : core.defaultBuildConfig;
    if (exports.config && exports.config.config) {
        exports.config.config = core.mergeConfig(buildConfig, exports.config.config);
    }
    store.buildConfig = (exports.config === null || exports.config === void 0 ? void 0 : exports.config.config) || buildConfig;
};

exports.addStoriesDoc = addStoriesDoc;
exports.getSerializedStore = getSerializedStore;
exports.initializeBuildOptions = initializeBuildOptions;
exports.removeStoriesDoc = removeStoriesDoc;
exports.reserveStories = reserveStories;
exports.store = store;
